const EventEmitter = require('events');
const { UserManager, ChannelManager, GuildManager, Constants, Util } = require('discord.js');
const { Events } = Constants;
const ClientPresence = require('../structures/ClientPresence');
const WebSocketManager = require('../websocket/Manager');

class Client extends EventEmitter {
  constructor(options) {
    super(Object.assign({ _tokenType: 'Bot' }, options));
      this.ws = new WebSocketManager(this);
      this.users = new UserManager(this);
      this.guilds = new GuildManager(this);
      this.channels = new ChannelManager(this);
      this.presence = new ClientPresence(this, this.options.presence);
      Object.defineProperty(this, 'token', { 
        writable: true 
      });
      if (!this.token && 'DISCORD_TOKEN' in process.env) {
        this.token = process.env.DISCORD_TOKEN;
      } else {
        this.token = null;
      }
  }

  run(token = this.token) {
    if (!token || typeof token !== 'string') throw new TypeError('Bot tokeni eksik.');
    this.token = token = token.replace(/^(Bot|Bearer)\s*/i, '');
    this.emit(
      Events.DEBUG,
      `Sağlanan jeton: ${token
        .split('.')
        .map((val, i) => (i > 1 ? val.replace(/./g, '*') : val))
        .join('.')}`,
    );

    if (this.options.presence) {
      this.options.ws.presence = this.presence._parse(this.options.presence);
    }

    this.emit(Events.DEBUG, 'Ağ geçidine bağlanmaya hazırlanıyor...');

    try {
      this.ws.connect();
      return this.token;
    } catch (error) {
      this.destroy();
      throw new Error(error);
    }
  }

  destroy() {
    super.destroy();
    this.ws.destroy();
    this.token = null;
  }

  createMessage(channelId, message) {
    if (!channelId) throw new Error('Hangi kanala mesaj atılcak?');
    if (!message) throw new Error('Bot boş mesaj gönderemez!?');
    return this.client.channels.cache.get(channelId).send(message).catch(async () => {
      throw new Error('Mesaj gönderilemedi.');
    });
  }
}

module.exports = Client;