const EventEmitter = require('events');
const { Util, Collection, Constants } = require('discord.js');
const { Events, ShardEvents, WSCodes, Status } = Constants;
const UNRECOVERABLE_CLOSE_CODES = Object.keys(WSCodes).slice(1).map(Number);
const UNRESUMABLE_CLOSE_CODES = [1000, 4006, 4007];

class Manager extends EventEmitter {
  constructor(client) {
    super(client);
      Object.defineProperty(this, 'client', { value: client });
      this.shards = new Collection();
      this.gateway = null;
      this.reconnecting = false;
      this.status = Status.IDLE;
      this.destroyed = false;
  }

  get ping() {
    const sum = this.shards.reduce((a, b) => a + b.ping, 0);
    return sum / this.shards.size;
  }

  debug(message, shard) {
    this.client.emit(Events.DEBUG, `[WS => ${shard ? `Shard ${shard.id}` : 'Manager'}] ${message}`);
  }

  async connect() {
    const invalidToken = "Bot jetonu yanlış veya yazılmamış.";
    const {
      url: gatewayURL,
      shards: recommendedShards,
      session_start_limit: sessionStartLimit,
    } = await this.client.api.gateway.bot.get().catch(() => {
      return console.error(invalidToken);
    });
    const { total, remaining } = sessionStartLimit;
    this.debug(`Getirilen Ağ Geçidi Bilgileri
    URL: ${gatewayURL}
    Önerilen Shards: ${recommendedShards}`);
    this.debug(`Oturum Sınırı Bilgisi
    Toplam: ${total}
    Kalan: ${remaining}`);
    this.gateway = `${gatewayURL}/`;
  }

  destroy() {
    if (this.destroyed) return;
    this.debug(`Manager yıkıldı:\n${new Error('MANAGER_DESTROYED').stack} tarafından arandı.`);
    this.destroyed = true;
    this.shardQueue.clear();
    for (const shard of this.shards.values()) {
      shard.destroy({ 
        closeCode: 1000, 
        reset: true, 
        emit: false, 
        log: false 
      });
    }
  }
}

module.exports = Manager;